from matplotlib import units
from pyqtgraph.metaarray.MetaArray import axis
from scipy.optimize import curve_fit
import numpy as np
from os import listdir
from scipy.stats import circmean
import sys
import json
from PyQt5.QtCore import Qt
from os.path import isfile, join
import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication, QHBoxLayout, QLabel, QSlider, QPushButton, \
    QVBoxLayout, QWidget


###
# Sisesta siia failinimi, kuhu on eeltöötluse programmi abil salvesatud sobitatud funktsioonide parameetrid sageduste kaupa.
###
filename = "naidismootmised/fit_data.json"


def read_fit_data():
    sagedused = []
    vaartused = []
    kalibratsioon = []
    with open(filename, 'r') as fitfile:
        for row in fitfile:
            pieces = row.split(';')
            sagedused.append(int(pieces[0]) * 1000)
            vaartused.append(json.loads(pieces[1]))
            kalibratsioon.append(circmean(np.array(json.loads(pieces[2])), axis = 1, low = -np.pi, high = np.pi))
            
    return sagedused, vaartused, kalibratsioon


def wrap(data):
    wrapping = 0
    return np.where(data <= np.pi, data, data - 2*np.pi)

def linear_fit_func(x, a, b):
    return a*x + b

def make_calibration_res_continous(calib_in):
    calib_out = np.zeros_like(calib_in)
    calib_out[0] = calib_in[0]
    for idx in range(1, calib_in.size):
        new_val = calib_in[idx]
        while abs(new_val - calib_out[idx - 1]) > np.pi:
            print(new_val, calib_out[idx - 1])
            print(idx, abs(new_val - calib_out[idx - 1]))
            if new_val > calib_out[idx - 1]:
                new_val -= 2 * np.pi
            else:
                new_val += 2 * np.pi
        calib_out[idx] = new_val
    return calib_out


class Slider(QWidget):
    def __init__(self, step_size, parent=None):
        super(Slider, self).__init__(parent=parent)
        self.verticalLayout = QVBoxLayout(self)
        self.label = QLabel(self)
        self.label.setFixedWidth(40)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.addWidget(self.label)
        self.slider = QSlider(self)
        self.slider.setOrientation(Qt.Horizontal)
        self.step_size = step_size
        self.slider.setValue(0)
        self.slider.setMinimum(0)
        self.slider.setMaximum(360 // step_size)
        self.horizontalLayout.addWidget(self.slider)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.resize(self.sizeHint())
        self.slider.valueChanged.connect(self.setLabelValue)
        self.x = None
        self.setLabelValue(self.slider.value())

    def setLabelValue(self, value):
        self.x = self.step_size * value
        self.label.setText("{0:.4g}".format(self.x))


class Widget(QWidget):
    def __init__(self, parent=None):
        super(Widget, self).__init__(parent=parent)
        self.verticalLayout = QVBoxLayout(self)

        self.win = pg.GraphicsWindow(title="Distance")
        self.verticalLayout.addWidget(self.win)
        current_plot = self.win.addPlot(title="Distance between two antennas")
        current_plot.setLabel('left', text="Distance (m)")
        current_plot.setLabel('bottom', text="Frequency", units="Hz")
        current_plot.addLegend()
        fit_pen1 = pg.mkPen('r', width=1, style=Qt.DotLine) 
        current_plot_curve1 = current_plot.plot(pen='r', name='Sla1-Sla2')
        current_fit_curve1 = current_plot.plot(pen=fit_pen1)
        current_plot_curve1.setData(sagedused, vaartused_normalized[:,0])
        current_fit_curve1.setData(sagedused, vaartused_fitted[:,0])
        fit_pen2 = pg.mkPen('b', width=1, style=Qt.DotLine)
        current_plot_curve2 = current_plot.plot(pen='b', name='Sla1-Mast')
        current_fit_curve2 = current_plot.plot(pen=fit_pen2)
        current_plot_curve2.setData(sagedused, vaartused_normalized[:,1])
        current_fit_curve2.setData(sagedused, vaartused_fitted[:,1])
        fit_pen3 = pg.mkPen('c', width=1, style=Qt.DotLine)
        current_plot_curve3 = current_plot.plot(pen='c', name='Sla2-Mast')
        current_fit_curve3 = current_plot.plot(pen=fit_pen3)
        current_plot_curve3.setData(sagedused, vaartused_normalized[:,2])
        current_fit_curve3.setData(sagedused, vaartused_fitted[:,2])

        self.win2 = pg.GraphicsWindow(title="Angle in coordinate system")
        self.verticalLayout.addWidget(self.win2)
        current_plot = self.win2.addPlot(title="Angle of antennas")
        current_plot.setLabel('left', text="Angle (rad)")
        current_plot.setLabel('bottom', text="Frequency", units="Hz")
        current_plot.addLegend()
        current_plot_curve1 = current_plot.plot(pen='r', name='Sla1-Sla2')
        current_fit_curve1 = current_plot.plot(pen='r')
        current_plot_curve1.setData(sagedused, vaartused[:, 0, 1])
        current_plot_curve2 = current_plot.plot(pen='b', name='Sla1-Mast')
        current_fit_curve2 = current_plot.plot(pen='b')
        current_plot_curve2.setData(sagedused, vaartused[:, 1, 1])
        current_plot_curve3 = current_plot.plot(pen='c', name='Sla2-Mast')
        current_fit_curve3 = current_plot.plot(pen='c')
        current_plot_curve3.setData(sagedused, np.where(vaartused[:, 2, 1] < 0, vaartused[:, 2, 1] + 2 * np.pi, vaartused[:, 2, 1]))
        
        self.win3 = pg.GraphicsWindow(title="Calibration")
        self.verticalLayout.addWidget(self.win3)
        current_plot = self.win3.addPlot(title="Calibration - additional phase shift in cables")
        current_plot.setLabel('left', text="Phase shift (rad)")
        current_plot.setLabel('bottom', text="Frequency", units="Hz")
        current_plot.addLegend()
        current_plot_curve1 = current_plot.plot(pen='r', name='Sla1-Sla2')
        current_fit_curve1 = current_plot.plot(pen='r')
        current_plot_curve1.setData(sagedused, kalibratsioon_continous[:, 0])
        current_plot_curve2 = current_plot.plot(pen='b', name='Sla1-Mast')
        current_fit_curve2 = current_plot.plot(pen='b')
        current_plot_curve2.setData(sagedused, kalibratsioon_continous[:, 1])
        current_plot_curve3 = current_plot.plot(pen='c', name='Sla2-Mast')
        current_fit_curve3 = current_plot.plot(pen='c')
        current_plot_curve3.setData(sagedused, kalibratsioon_continous[:, 2])


if __name__ == '__main__':
    pg.setConfigOption('background', 'w')
    pg.setConfigOption('foreground', 'k')

    sagedused, vaartused, kalibratisoon = read_fit_data()
    
    c = 299792458
    sagedused = np.array(sagedused)
    lainepikkused = c / (sagedused * 1000)
    vaartused = np.array(vaartused)
    vaartused_normalized = np.zeros(vaartused.shape[:-1])
    vaartused_normalized[:, 0] = vaartused[:, 0, 0] * lainepikkused / np.pi / 2
    vaartused_normalized[:, 1] = vaartused[:, 1, 0] * lainepikkused / np.pi / 2
    vaartused_normalized[:, 2] = vaartused[:, 2, 0] * lainepikkused / np.pi / 2

    popt1, _ = curve_fit(linear_fit_func, sagedused, vaartused_normalized[:, 0])
    popt2, _ = curve_fit(linear_fit_func, sagedused, vaartused_normalized[:, 1])
    popt3, _ = curve_fit(linear_fit_func, sagedused, vaartused_normalized[:, 2])

    vaartused_fitted = np.zeros_like(vaartused_normalized)
    vaartused_fitted[:, 0] = popt1[0] * sagedused + popt1[1]
    vaartused_fitted[:, 1] = popt2[0] * sagedused + popt2[1]
    vaartused_fitted[:, 2] = popt3[0] * sagedused + popt3[1]

    print(popt1, popt2, popt3)

    kalibratisoon = np.array(kalibratisoon)
    print(kalibratisoon[:, 0])

    kalibratsioon_difference = np.array((wrap(vaartused[:, 0, 2] - kalibratisoon[:, 0]), wrap(vaartused[:, 1, 2] - kalibratisoon[:, 1]), wrap(vaartused[:, 2, 2] - kalibratisoon[:, 2]))).T
    kalibratsioon_continous = np.array((make_calibration_res_continous(kalibratsioon_difference[:, 0]), make_calibration_res_continous(kalibratsioon_difference[:, 1]), make_calibration_res_continous(kalibratsioon_difference[:, 2]))).T
    
    app = QApplication(sys.argv)
    w = Widget()
    w.show()
    sys.exit(app.exec_())