from scipy.optimize import curve_fit
import numpy as np
from os import listdir
import sys
import json
from PyQt5.QtCore import Qt
from os.path import isfile, join
import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication, QHBoxLayout, QLabel, QSlider, QFrame, QSizePolicy, QPushButton, QTextEdit, \
    QVBoxLayout, QWidget


#####
# Programm süsteemi iseloomustamiseks ringikujuliselt kogutud andmete eeltõõtluseks ühe sageduse jaoks.
# Eeldab sisendina csv faili, mis sisaldab tulpi nurgaga ja kolme antennipaari measured faasinihetega.
# Väljundina on võimalik savestada samasse kausta signaal, millel on faasi pakkimise parandused tehtud ja ka fitted funktsiooni parameetrid.
#####

# Failinimi, mille töötlemisega tegeletakse
filename = "naidismootmised/karakter_naidis_sag_54000kHz.csv"

# Failinimi, kuhu salvestatakse fitted funktsioonide parameetrid sageduse jaoks
filename_out = "naidismootmised/fit_data.json"

pg.setConfigOption('background', 'w')
pg.setConfigOption('foreground', 'k')


def fit_func(x, a, c, d):
    # Sinusoidaalne funktsioon mida sobitatakse. a, b, c on otsitavad parameetrid.
    return a*np.sin(2*np.pi/360*x + c) + d


def save_fixed_data():
    # Faasiparanudsega signaali salvestamine. Kuju jääb samaks nagu algsel failil ja seda on võimalik omakorda käesoleva programmiga avada.
    with open(filename.split('.')[0] + "_unwrapped.csv", 'w') as outfile:
        for i in range(len(calibs[0])):
            outfile.write('calib,' + str(calibs[0][i]) + ',' + str(calibs[1][i]) + ',' + str(calibs[2][i]) + '\n')
        for i in range(len(fixed_vaartused[0])):
            outfile.write(str(int(kraadid[i])) + ',' + str(fixed_vaartused[0][i]) + ',' + str(fixed_vaartused[1][i]) + ',' + str(fixed_vaartused[2][i]) + '\n')

def save_fit_data():
    # Lisab etteantud faili sageduse jaoks fitted parameetrid.
    with open(filename_out, 'a') as fitfile:
        fitfile.write(filename.split("_")[3].split("kHz")[0] + ";" + json.dumps(parameters_fitted.tolist()) + ";" + json.dumps(calibs) + "\n")


class Slider(QWidget):
    # Liuguri klass GUI elemendi jaoks.
    def __init__(self, step_size, parent=None):
        super(Slider, self).__init__(parent=parent)
        self.verticalLayout = QVBoxLayout(self)
        self.label = QLabel(self)
        self.label.setFixedWidth(40)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.addWidget(self.label)
        self.slider = QSlider(self)
        self.slider.setOrientation(Qt.Horizontal)
        self.step_size = step_size
        self.slider.setValue(0)
        self.slider.setMinimum(0)
        self.slider.setMaximum(360 // step_size)
        self.horizontalLayout.addWidget(self.slider)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.resize(self.sizeHint())
        self.slider.valueChanged.connect(self.setLabelValue)
        self.x = None
        self.setLabelValue(self.slider.value())

    def setLabelValue(self, value):
        self.x = self.step_size * value
        self.label.setText("{0:.4g}".format(self.x))


class Widget(QWidget):
    # Graafilise liidese definitsioon.
    def __init__(self, parent=None):
        super(Widget, self).__init__(parent=parent)
        self.horizontalLayout = QHBoxLayout(self)
        self.buttonLayout = QVBoxLayout(self)


        self.sliders = []
        self.plot_curves = []
        self.fit_curves = []
        
        for plot_index in range(3):
            plot_layout = QVBoxLayout(self)

            self.win = pg.GraphicsWindow(title="Sla1-Sla2")
            plot_layout.addWidget(self.win)
            current_plot = self.win.addPlot(title="Sla1-Sla2")
            current_plot.addLegend()
            current_plot.setLabel('bottom', 'Signaali nurk (kraadi)')
            current_plot.setLabel('left', 'Hilistus faasina (rad)')
            current_plot_curve = current_plot.plot(pen='r', name='measured')
            current_fit_curve = current_plot.plot(pen='g', name='fitted')
            self.plot_curves.append(current_plot_curve)
            self.fit_curves.append(current_fit_curve)
            self.horizontalLayout.addLayout(plot_layout)
            plot_sliders = []
            for _ in range(8):
                current_slider = Slider(15)
                current_slider.slider.valueChanged.connect(self.update_data)
                plot_sliders.append(current_slider)
                plot_layout.addWidget(current_slider)
            self.sliders.append(plot_sliders)
        
        button1 = QPushButton(self)
        button1.setText("Save\nunwrapped")
        button1.clicked.connect(save_fixed_data)
        button1.setEnabled(True)

        button2 = QPushButton(self)
        button2.setText("Fit")
        button2.clicked.connect(self.fit_all)
        button2.setEnabled(True)

        button3 = QPushButton(self)
        button3.setText("Save\nfitted")
        button3.clicked.connect(save_fit_data)
        button3.setEnabled(True)

        self.buttonLayout.addWidget(button1)
        self.buttonLayout.addWidget(button2)
        self.buttonLayout.addWidget(button3)
        self.horizontalLayout.addLayout(self.buttonLayout)
        self.update_data()

    def fit_all(self):
        global parameters_fitted, vaartused_fitted
        popt1, _ = curve_fit(fit_func, kraadid, fixed_vaartused[0], bounds=([0., -np.pi, -20.], [20., np.pi, 20.]))
        popt2, _ = curve_fit(fit_func, kraadid, fixed_vaartused[1], bounds=([0., -np.pi, -20.], [20., np.pi, 20.]))
        popt3, _ = curve_fit(fit_func, kraadid, fixed_vaartused[2], bounds=([0., -np.pi, -20.], [20., np.pi, 20.]))
        vaartused_fitted1 = fit_func(kraadid, popt1[0], popt1[1], popt1[2])
        vaartused_fitted2 = fit_func(kraadid, popt2[0], popt2[1], popt2[2])
        vaartused_fitted3 = fit_func(kraadid, popt3[0], popt3[1], popt3[2])
        vaartused_fitted = [vaartused_fitted1, vaartused_fitted2, vaartused_fitted3]
        parameters_fitted = np.array([popt1, popt2, popt3])
        self.fit_curves[0].setData(kraadid, vaartused_fitted1)
        self.fit_curves[1].setData(kraadid, vaartused_fitted2)
        self.fit_curves[2].setData(kraadid, vaartused_fitted3)


    def update_data(self):
        global fixed_vaartused
        fixed_vaartused = vaartused.copy()
        for plot_idx, plot_curve in enumerate(self.plot_curves):
            for slider_idx in range(4):
                start = self.sliders[plot_idx][slider_idx * 2].x
                end = self.sliders[plot_idx][slider_idx * 2 + 1].x
                if start < end:
                    kraadid_range = np.where((kraadid >= start) & (kraadid < end))
                elif end < start:
                    kraadid_range = np.where((kraadid < end) | (kraadid >= start))
                    print(kraadid_range)
                else:
                    kraadid_range = np.array([], dtype=np.uint32)
                fixed_vaartused[plot_idx][kraadid_range] += np.pi * 2
            plot_curve.setData(kraadid, fixed_vaartused[plot_idx])
                

if __name__ == '__main__':

    kraadid = []
    calibs = [[], [], []]
    vaartused = [[], [], []]
    vaartused_fitted = None
    parameters_fitted = None
    i = 0

    with open(filename, 'r') as file:
        for rida in file:
            tykid = rida.split(',')
            if(tykid[0].isnumeric()):
                kraadid.append(float(tykid[0]))
                vaartused[0].append(float(tykid[1]))
                vaartused[1].append(float(tykid[2]))
                vaartused[2].append(float(tykid[3]))
            elif tykid[0] == 'calib':
                calibs[0].append(float(tykid[1]))
                calibs[1].append(float(tykid[2]))
                calibs[2].append(float(tykid[3]))
            i+=1

    

    kraadid = np.array(kraadid)
    vaartused = np.array(vaartused)
    fixed_vaartused = vaartused.copy()

    app = QApplication(sys.argv)
    w = Widget()
    w.show()
    sys.exit(app.exec_())