import numpy as np
from scipy.signal import lfilter, butter
import datetime, os

#### TO BE REPLACED BY CMDLINE ARGUMENTS ####
SIGNAL_FREQ           = 53.1e6
LO_FREQ               = 53e6
SAMPLE_FREQ_END       = 2e6
ANTENNA_COORDS_M_X    = [0.0, -1.3, 1.3]#[0.0, 1.5, 0.0]
ANTENNA_COORDS_M_Y    = [1.5, -0.75, -0.75]#[0.0, 0.0, 2.0]
SIGNAL_SOURCE_ANGLES  = np.arange(0, 360, 1)
NUM_SYNC_SAMPLES      = 10000
NUM_TOTAL_SAMPLES     = 100000
MEASUREMENT_NOISE     = 0
HIGHER_THAN_SAMP_MULT = 100
####  ---------------------------------  ####


####        CONVENIENCE SUBRESULTS       ####
SAMPLE_FREQ = SAMPLE_FREQ_END * HIGHER_THAN_SAMP_MULT
SPEED_OF_LIGHT        = 3e8
SAMPLING_PERIOD       = 1.0 / SAMPLE_FREQ
TOTAL_SAMPLING_TIME   = NUM_TOTAL_SAMPLES * HIGHER_THAN_SAMP_MULT / SAMPLE_FREQ
ANTENNA_COORDS_M_X    = np.array(ANTENNA_COORDS_M_X)
ANTENNA_COORDS_M_Y    = np.array(ANTENNA_COORDS_M_Y)
####  ---------------------------------  ####


sampling_times = np.arange(0, TOTAL_SAMPLING_TIME, SAMPLING_PERIOD)
#print(sampling_times.shape)

folder_name = datetime.date.today().strftime("%y%m%d") + "_" + str(int(SIGNAL_FREQ)) + "Hz"
folder_name = "data"
if not os.path.isdir(folder_name):
    os.makedirs(folder_name)

for signal_source_angle in SIGNAL_SOURCE_ANGLES:

    # Simulate random phase shifts on startup with generating random phase shift to LO waveform
    lo_phase_shifts = np.random.random(3) * np.pi * 2
    print('lo: ', lo_phase_shifts)
    print('diffs: ', lo_phase_shifts[0] - lo_phase_shifts[1], lo_phase_shifts[1] - lo_phase_shifts[2], lo_phase_shifts[2] -  lo_phase_shifts[0])

    lo_sin_values = np.sin(2 * np.pi * LO_FREQ * np.tile(sampling_times, (3, 1)).T + lo_phase_shifts).T
    lo_cos_values = np.cos(2 * np.pi * LO_FREQ * np.tile(sampling_times, (3, 1)).T + lo_phase_shifts).T

    signal_source_coord_x = 100 * np.cos(np.radians(signal_source_angle))
    signal_source_coord_y = 100 * np.sin(np.radians(signal_source_angle))

    distances_from_source = np.sqrt(np.power(ANTENNA_COORDS_M_X - signal_source_coord_x, 2) + np.power(ANTENNA_COORDS_M_Y - signal_source_coord_y, 2))
    times_from_source = distances_from_source / SPEED_OF_LIGHT

    signal_values = np.sin(2 * np.pi * SIGNAL_FREQ * (np.tile(sampling_times, (3, 1)).T - times_from_source)).T
    #print(signal_values.shape)
    signal_values[:, :NUM_SYNC_SAMPLES * HIGHER_THAN_SAMP_MULT] = np.sin(2 * np.pi * SIGNAL_FREQ * sampling_times[:NUM_SYNC_SAMPLES * HIGHER_THAN_SAMP_MULT])
    mixed_down_values = np.array([signal_values * lo_sin_values, signal_values * lo_cos_values])
    b, a = butter(4, 0.03)
    mixed_down_values = lfilter(b, a, mixed_down_values)
    mixed_down_values = mixed_down_values / np.max(np.abs(mixed_down_values)) * 125
    mixed_down_values.astype(np.int8)
    mixed_down_values_interweaved = np.empty((3, NUM_TOTAL_SAMPLES * 2), dtype=np.float32)
    mixed_down_values_interweaved[:, 0::2] = mixed_down_values[0, :, ::HIGHER_THAN_SAMP_MULT]
    mixed_down_values_interweaved[:, 1::2] = mixed_down_values[1, :, ::HIGHER_THAN_SAMP_MULT]
    #print(mixed_down_values_interweaved.shape)

    print(mixed_down_values_interweaved.shape)
    mixed_down_values_interweaved[0].tofile(folder_name + "/" + "dev0_" + str(signal_source_angle) + ".bin")
    mixed_down_values_interweaved[1].tofile(folder_name + "/" + "dev1_" + str(signal_source_angle) + ".bin")
    mixed_down_values_interweaved[2].tofile(folder_name + "/" + "dev2_" + str(signal_source_angle) + ".bin")

