from scipy.signal import firwin
from scipy.special.orthogonal import jacobi
from scipy.stats import circmean
import numpy as np

from pyargus.directionEstimation import gen_ula_scanning_vectors, corr_matrix_estimate, DOA_plot, gen_scanning_vectors
from pyargus.directionEstimation import DOA_Bartlett, DOA_Capon, DOA_LPM, DOA_MEM, DOA_MUSIC


def fir_filter(x, taps, decim=1):
    x2 = np.pad(x, (len(taps)-1, 0))
    y = np.zeros_like(x2)
    for i in range(0, len(x), decim):
        yi = np.sum(np.flip(taps) * x2[i:i+len(taps)])
        y[i] = yi
    return y


def sig_source_c(samp_rate, freq, amp, N):
    t = np.arange(N) / samp_rate
    y = np.cos(2. * np.pi * freq * t) + 1j * np.sin(2. * np.pi * freq * t)
    return y


def mix(lo, data, phase=0.0):
    min_len = np.min((len(lo), len(data)))
    cphase = np.exp(1j * phase)
    y = lo[:min_len] * data[:min_len] * cphase
    return np.array(y)


SAMPLING_RATE = 2000000

FILTER_CUTOFF = 10000
DECIM = 10

ANTENNA_COORDS_M_X    = [0.0, -1.3, 1.3]#[0.0, 1.5, 0.0]
ANTENNA_COORDS_M_Y    = [1.5, -0.75, -0.75]#[0.0, 0.0, 2.0]
ANTENNA_COORDS_M_X = np.array(ANTENNA_COORDS_M_X)
ANTENNA_COORDS_M_Y = np.array(ANTENNA_COORDS_M_Y)

NUM_SYNC_SAMPLES = 10000
BUFFER1 = 1000
BUFFER2 = 2000

taps = firwin(97, 20000/2000000)
folder_name = "dist_diff"

probabilities_array = []

distances = [2 ** n for n in range(30)]

for i in distances:

    data1 = np.fromfile(folder_name + "/dev0_45_" + str(i) + ".bin", dtype=np.float32)
    data1_i = data1[0::2].astype(np.float32)
    data1_q = data1[1::2].astype(np.float32)
    data1 = data1_i + data1_q * 1j

    data2 = np.fromfile(folder_name + "/dev1_45_" + str(i) + ".bin", dtype=np.float32)
    data2_i = data2[0::2].astype(np.float32)
    data2_q = data2[1::2].astype(np.float32)
    data2 = data2_i + data2_q * 1j

    data3 = np.fromfile(folder_name + "/dev2_45_" + str(i) + ".bin", dtype=np.float32)
    data3_i = data3[0::2].astype(np.float32)
    data3_q = data3[1::2].astype(np.float32)
    data3 = data3_i + data3_q * 1j

    all_data_sync = [data1[:NUM_SYNC_SAMPLES - BUFFER1],
                    data2[:NUM_SYNC_SAMPLES - BUFFER1], data3[:NUM_SYNC_SAMPLES - BUFFER1]]
    all_data_signal = [data1[NUM_SYNC_SAMPLES + BUFFER2:],
                    data2[NUM_SYNC_SAMPLES + BUFFER2:], data3[NUM_SYNC_SAMPLES + BUFFER2:]]
    avg_phases = []
    avg_phases_raw = []


    lo = sig_source_c(SAMPLING_RATE, -FILTER_CUTOFF, 1, len(data1))
    phase = -np.pi * FILTER_CUTOFF / SAMPLING_RATE * (len(taps)-1)

    diffs_sync = [
        circmean(np.angle(all_data_sync[0]) - np.angle(all_data_sync[1])),
        circmean(np.angle(all_data_sync[1]) - np.angle(all_data_sync[2])),
        circmean(np.angle(all_data_sync[2]) - np.angle(all_data_sync[0]))
    ]

    diff_sync1 = circmean(np.angle(all_data_sync[1]) - np.angle(all_data_sync[0]))
    diff_sync2 = circmean(np.angle(all_data_sync[2]) - np.angle(all_data_sync[0]))

    print('sync_diffs raw: ', diffs_sync)

    all_data_sync_fil = []
    for data in all_data_sync:
        despun = mix(lo, data, phase=phase)
        filtered_data = fir_filter(despun, taps, DECIM)
        all_data_sync_fil.append(filtered_data)

    diffs_sync = [
        circmean(np.angle(all_data_sync_fil[0]) - np.angle(all_data_sync_fil[1])),
        circmean(np.angle(all_data_sync_fil[1]) - np.angle(all_data_sync_fil[2])),
        circmean(np.angle(all_data_sync_fil[2]) - np.angle(all_data_sync_fil[0]))
    ]

    print('sync_diffs fil: ', diffs_sync)

    all_data_signal_fil = []
    for data in all_data_signal:
        despun = mix(lo, data, phase=phase)
        filtered_data = fir_filter(despun, taps, DECIM)
        all_data_signal_fil.append(filtered_data)

    '''
    diffs_signal = [
        circmean(np.angle(all_data_signal_fil[0]) -
                np.angle(all_data_signal_fil[1])),
        circmean(np.angle(all_data_signal_fil[1]) -
                np.angle(all_data_signal_fil[2])),
        circmean(np.angle(all_data_signal_fil[2]) -
                np.angle(all_data_signal_fil[0]))
    ]
    print('signal_diffs fil: ', diffs_signal)

    diffs_signal_fixed = np.array(diffs_signal) - np.array(diffs_sync)

    print('pure signal diffs: ', diffs_signal_fixed)
    '''
    print(type(all_data_signal_fil))
    fixed_data1 = all_data_signal_fil[1] * np.exp(0-diff_sync1*1j)
    fixed_data2 = all_data_signal_fil[2] * np.exp(0-diff_sync2*1j)
    fixed_signal_data = [all_data_signal_fil[0], fixed_data1, fixed_data2]
    
    
    diffs_fixed = [
        circmean(np.angle(fixed_signal_data[0]) -
                np.angle(fixed_signal_data[1])),
        circmean(np.angle(fixed_signal_data[1]) -
                np.angle(fixed_signal_data[2])),
        circmean(np.angle(fixed_signal_data[2]) -
                np.angle(fixed_signal_data[0]))
    ]

    print('fixe signal diffs: ', diffs_fixed)


np.save("phase_difs_dist_diff", diffs_fixed[0])

