import numpy as np
import pyqtgraph as pg

ANTENNA_COORDS_M_X    = [0.0, 0.0]
ANTENNA_COORDS_M_Y    = [1.0, -1.0]
SIGNAL_SOURCE_ANGLES  = np.arange(0, 360, 1)

signal_source_coords_x = []
signal_source_coords_y = []

for signal_source_angle in SIGNAL_SOURCE_ANGLES:
    signal_source_coord_x = 15 * np.cos(np.radians(signal_source_angle))
    signal_source_coord_y = 15 * np.sin(np.radians(signal_source_angle))
    signal_source_coords_x.append(signal_source_coord_x)
    signal_source_coords_y.append(signal_source_coord_y)

pg.setConfigOption('background', 'w')
pg.setConfigOption('foreground', 'k')
win = pg.GraphicsLayoutWidget(show=True, title="Debug", size=(1100, 340))

win.ci.layout.setColumnFixedWidth(0, 600)
win.ci.layout.setColumnFixedWidth(1, 300)
win.ci.layout.setColumnFixedWidth(2, 120)
win.ci.layout.setRowFixedHeight(0, 320)


p1 = win.addPlot()
p2 = win.addPlot()
p3 = win.addPlot()

data = np.load("probabilities.npy")

p1.setLabel('left', 'Probability of correct angle')
p1.setLabel('bottom', 'Angle', units='deg')
p1.setTitle('Probability of the correct angle in the output of the Capon algorithm')

p2.setTitle('Probability shown in 2D space')
p2.setLabel('left', 'y', units='m')
p2.setLabel('bottom', 'x', units='m')
p2.getAxis('left').setTicks([[(15, "100"), (0,"0"), (-15, "-100")]])
p2.getAxis('bottom').setTicks([[(15, "100"), (0,"0"), (-15, "-100")]])

p3.setTitle(' ')

p3.getAxis('bottom').setPen(pg.mkPen('w'))
p3.getAxis('left').setPen(pg.mkPen('w'))
p3.getAxis('bottom').setTextPen(pg.mkPen('w'))
p3.getAxis('left').setTextPen(pg.mkPen('w'))

p1.plot(data, pen='d')
c2 = p2.plot()

min_prob = np.min(data)
max_prob = np.max(data)
cm = pg.colormap.get('CET-R2') # prepare a diverging color map
#cm.setMappingMode('diverging') # set mapping mode
#cm = pg.colormap.get(            'CET-CBL2')  # Use another if colour scheme is not good for you: https://colorcet.holoviz.org/user_guide/
cm_colours = cm.getColors()
print(len(cm_colours))
brush_colours = [pg.mkColor(cm_colours[int(s)]) for s in (data - min_prob) / (max_prob - min_prob) * 255]
brush_colours_antennas = [pg.mkColor(0,0,0), pg.mkColor(0,0,0)]
gl = pg.GradientLegend((10,220), (0,20))
gl.pen = pg.mkPen('w')
gl.setColorMap(cm)

x = np.arange(len(data))
scatter = pg.ScatterPlotItem()
scatter.setData(ANTENNA_COORDS_M_X + signal_source_coords_x[::4],
                ANTENNA_COORDS_M_Y + signal_source_coords_y[::4], size=6, hoverable=True, hoverSymbol='s',
                        symbol='o', pen=None, brush=brush_colours_antennas + brush_colours[::4])
p2.addItem(scatter)
gl.setLabels({"{0:.3f}".format(min_prob):0, "{0:.3f}".format(max_prob) :1})
p3.addItem(gl)

pg.QtGui.QApplication.exec_()

