
const WebSocket = require('ws');
const http = require('http');
var express = require('express');
const yargs = require('yargs');
const { spawn } = require('child_process');
const { strict } = require('yargs');


const app = express();

//initialize a simple http server
const server = http.createServer(app);

//initialize the WebSocket server instance
const wss = new WebSocket.Server({ server });

var last_send_time = Date.now();

const argv = yargs
    .option('location-default', {
        alias: 'l',
        description: 'For running the program without location input. Makes all the locations default to Tartu, with x-axis pointing straight to east',
        type: 'boolean',
    })
    .help()
    .alias('help', 'h')
    .argv;


function send_new_info_to_all_ws_clients(info_json){
        wss.clients.forEach(ws => {
            ws.send(JSON.stringify(info_json));
        });
        last_send_time = Date.now();
    
}

app.use(
    express.urlencoded({
        extended: true,
        parameterLimit: 5000
    })
)

app.use(express.json())

app.post('/', function (req, res) {
    console.log(req.body);
    res.send('POST request to the homepage');
    send_new_info_to_all_ws_clients(req.body)
})

clients = []
    
wss.on('connection', ws => {
    console.log('connection');
    if(clients.length == 0){
        start_scan();
    }
    clients.push(ws);
    //connection is up, let's add a simple simple event
    ws.on('message', message => {
        parsed_message = JSON.parse(message)
        //log the received message and send it back to the client
        console.log('received: %s', message);
        if(parsed_message['cmd'] == 'dir_find'){
            terminate_child();
            start_dir_finding(10);
        }
        if(parsed_message['cmd'] == 'scan'){
            terminate_child();
            start_scan();
        }
        
    });

    ws.on('close', message => {
        const index = clients.indexOf(ws);
        if (index > -1) {
            clients.splice(index, 1);
        }
        if(clients.length == 0){
            terminate_child();
        }
    });
});

//start our server
server.listen(process.env.PORT || 5678, () => {
    console.log(`Server started on port ${server.address().port}`);
});


function terminate_child(){
    if(child){
        child.kill();
    }
    child = null;
    data_buf = "";
}

function start_scan(){
    console.log('child exist: ')
    console.log((!child))
    if(!child){
        child = spawn('hackrf_sweep', ["-f " + min_freq / 1000000 +":" + max_freq / 1000000, "-w " + bin_width, "-n 8192"]);

        child.stdout.on('data', data => {
            split_data = data.toString().split("\n");
            split_data[0] = data_buf + split_data[0];
            data_buf = split_data[split_data.length - 1];
            for (var i = 0; i < split_data.length - 1; i++) {
                split_line = split_data[i].split(", ");
                current_freq = parseInt(split_line[2]);
                freq_index = (current_freq - min_freq) / bin_width
                if (last_freq - current_freq > 1000000){
                    send_counter -= 1;
                    if(!send_counter){
                        send_new_info_to_all_ws_clients({type: 'scan', data: scan_results})
                        count_here += 1;
                        send_counter = send_interval;
        
                        scan_results = Array((max_freq - min_freq) / bin_width);
        
                    }
                    //console.log(scan_results)
                }
                //console.log(current_freq)
                for(var j = 0; j < 5000000 / bin_width; j ++){
                    if(scan_results[freq_index + j]){
                        scan_results[freq_index + j] = {x: current_freq + j * bin_width, y:Math.max(parseFloat(split_line[6 + j]), scan_results[freq_index + j]['y'])};
                    }
                    else{
                        scan_results[freq_index + j] = {x: current_freq + j * bin_width, y:parseFloat(split_line[6 + j])};
                    }
                }
                last_freq = current_freq;
            }
        });
        


        child.stderr.on('data', data => {
            console.error(`stderr: ${data}`);
            console.log(count_here)
        });
        
        child.on('error', (error) => {
            console.error(`error: ${error.message}`);
        });
        
        child.on('close', (code) => {
            console.log(`child process exited with code ${code}`);
        });
    }
}

function start_dir_finding(frequency){
    child = spawn('python3', ["dir_finder/dummy_direction_finder.py"]);

    child.stdout.on('data', data => {
        console.log(data.length)
        split_data = data.toString().split("\n");
        send_new_info_to_all_ws_clients({type: 'dir', data: split_data[0]})
    });

    child.stderr.on('data', data => {
        console.error(`stderr: ${data}`);
    });
    
    child.on('error', (error) => {
        console.error(`error: ${error.message}`);
    });
    
    child.on('close', (code) => {
        console.log(`child process exited with code ${code}`);
    });
}

min_freq = 30000000;
max_freq = 90000000;
bin_width = 25000;

child = null;
data_buf = "";
last_freq = 0;
count_here = 0;
send_interval = 10;
send_counter = send_interval;

var scan_results = Array((max_freq - min_freq) / bin_width);
