Math.radians = function(degrees) {
    return degrees * Math.PI / 180;
}

Math.degrees = function(radians) {
    return radians * 180 / Math.PI;
}

function get_lat_lon_of_degrees(org_lat, org_lon, dist, degrees){
    degrees = degrees - 90;
    var R = 6378.1;
    var orig_lat_rad = Math.radians(org_lat);
    var orig_lon_rad = Math.radians(org_lon);
    var dir_rad = Math.radians(-degrees);

    end_lat = Math.asin( Math.sin(orig_lat_rad)*Math.cos(dist/R) + Math.cos(orig_lat_rad)*Math.sin(dist/R)*Math.cos(dir_rad))

    end_lon = orig_lon_rad + Math.atan2(Math.sin(dir_rad)*Math.sin(dist/R)*Math.cos(orig_lat_rad), Math.cos(dist/R)-Math.sin(orig_lat_rad)*Math.sin(end_lat))

    return [Math.degrees(end_lat), Math.degrees(end_lon)];
}

function draw_antennas(node_number, direction_degrees, org_lat, org_lon, antenna_distance_multiplier = 1){
    node_pos_dict = ant_locations[node_number];
    dividor = 1000.0 / antenna_distance_multiplier;
    x1_km = node_pos_dict.x1 / dividor;
    y1_km = node_pos_dict.y1 / dividor;
    x2_km = node_pos_dict.x2 / dividor;
    y2_km = node_pos_dict.y2 / dividor;
    x3_km = node_pos_dict.x3 / dividor;
    y3_km = node_pos_dict.y3 / dividor;
    point1_len = Math.sqrt(Math.pow(y1_km, 2) + Math.pow(x1_km, 2));
    point2_len = Math.sqrt(Math.pow(y2_km, 2) + Math.pow(x2_km, 2));
    point3_len = Math.sqrt(Math.pow(y3_km, 2) + Math.pow(x3_km, 2));
    point1_angle = Math.atan2(y1_km, x1_km) + Math.radians(direction_degrees);
    point2_angle = Math.atan2(y2_km, x2_km) + Math.radians(direction_degrees);
    point3_angle = Math.atan2(y3_km, x3_km) + Math.radians(direction_degrees);
    console.log("Ant_pos_coords:" + point1_angle + "; " + point2_angle + "; " + point3_angle + "\n");

    point1_coords = get_lat_lon_of_degrees(org_lat, org_lon, point1_len, Math.degrees(point1_angle));
    point2_coords = get_lat_lon_of_degrees(org_lat, org_lon, point2_len, Math.degrees(point2_angle));
    point3_coords = get_lat_lon_of_degrees(org_lat, org_lon, point3_len, Math.degrees(point3_angle));

    L.circle(point1_coords, {radius: 0.1, color: 'red'}).addTo(marker_layer);
    L.circle(point2_coords, {radius: 0.1, color: 'orange'}).addTo(marker_layer);
    L.circle(point3_coords, {radius: 0.1, color: 'blue'}).addTo(marker_layer);

    dir1 = direction_degrees - 34 + 90;
    dir2 = direction_degrees + 34 + 90;
    dir3 = -146 + direction_degrees + 90;
    dir4 = 146 + direction_degrees + 90;

    poly_coords2 = get_lat_lon_of_degrees(org_lat, org_lon, antenna_distance_multiplier * 0.0024, dir2);
    poly_coords3 = get_lat_lon_of_degrees(org_lat, org_lon, antenna_distance_multiplier * 0.0024, dir3);
    poly_coords4 = get_lat_lon_of_degrees(org_lat, org_lon, antenna_distance_multiplier * 0.0024, dir4);
    poly_coords1 = get_lat_lon_of_degrees(org_lat, org_lon, antenna_distance_multiplier * 0.0024, dir1);
    poly_coords = [poly_coords1, poly_coords2, poly_coords4, poly_coords3, poly_coords1];
    console.log("rect_loc" + poly_coords1 + ' ' + poly_coords2 + ' ' + dir1 + ' ' + dir2);
    new_map_object_dir = L.polyline(poly_coords, {color: 'black', weight: 1}).addTo(marker_layer);
}

var origlat = 58.36749340487921; //kva hoov: 58.37305627524618, 26.722288594483437, est centr: 59.0, 26.0 // põld: lat: 58.36749340487921, lon: 26.689747309201238
var origlon = 26.689747309201238;

var map = L.map('map').setView([origlat, origlon], 18);

mapLink = '<a href="http://openstreetmap.org">OpenStreetMap</a>';

L.tileLayer(
    'http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
    attribution: '&copy; ' + mapLink + ' Contributors',
    minZoom: 5,
    maxZoom: 19,
}).addTo(map);

var marker_layer = L.layerGroup([]);
marker_layer.addTo(map);

function get_distribution_poly_coords(distribution, org_lat, org_lon){
    if(distribution.length != 360){
        return;
    }
    latlngs_list = [];
    for(i = 0; i < 360; i++){
        latlngs_list.push(get_lat_lon_of_degrees(org_lat, org_lon, distribution[i] / 40 + 0.000, i));
    }
    return latlngs_list
}

function updateMap(data){
    marker_layer.clearLayers();
    parsed_data_list = JSON.parse(data)

    dist_latlngs_list = get_distribution_poly_coords(parsed_data_list, origlat, origlon);
    console.log(dist_latlngs_list)
    new_map_object_dir = L.polyline(dist_latlngs_list, {color: 'red', weight: 2}).addTo(marker_layer);
    new_map_object_dir.bindTooltip("Sagedus: " + freq + "<br>Viimati kuuldud: " + info.time + "<br>Võimsus: " + info.pow + "<br><br>" + "Mingi lisainfo");
    
}

function activate_scanning_mode(){
    data = {'cmd':'scan'};
	connection.send(JSON.stringify(data));
    activate_scan_view();
}

document.getElementById("stop_dir_find").onclick = activate_scanning_mode;