"""
Embedded Python Blocks:

Each time this file is saved, GRC will instantiate the first class it finds
to get ports and parameters of your block. The arguments to __init__  will
be the parameters. All of them are required to have default values!
"""

import numpy as np
from gnuradio import gr


class blk(gr.decim_block):  # other base classes are basic_block, decim_block, interp_block
    """Embedded Python Block example - a simple multiply const"""

    def __init__(self):  # only default arguments here
        """arguments to this function show up as parameters in GRC"""
        gr.decim_block.__init__(
            self,
            name='FFT find phase',   # will show up in GRC
            in_sig=[np.complex64],
            out_sig=[np.float32],
            decim=1024
        )
        # if an attribute with the same name as a parameter is found,
        # a callback is registered (properties work, too).

    def work(self, input_items, output_items):
        """example: multiply with constant"""
        in_len = len(input_items[0])
        if in_len < 1024:
            return 0
        else:
            for i in range(in_len // 1024):
                fft_res = np.fft.fft(input_items[0][i*1024:(i+1)*1024])
                fft_phases = np.angle(fft_res)
                output_items[0][i] = fft_phases[10]
            return i
