import numpy as np
from gnuradio import gr

def map_range( x,  in_min,  in_max,  out_min,  out_max):
    return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min


class complex_to_hackrf_complex(gr.sync_block):
    """
    This block limits complex signal resolution to 8 bits per ADC
    """
    def __init__(self):
        gr.sync_block.__init__(self,
            name="Complex to Hackrf",
            in_sig=[np.complex64],
            out_sig=[np.complex64]
        )


    def work(self, input_items, output_items):
        # clip input signals to [-1, 1] range
        clip_real = np.clip(np.real(input_items[0]),-1, 1)
        clip_imag = np.clip(np.imag(input_items[0]),-1, 1)
        # scale inputs to signed byte range
        real = map_range(clip_real, -1, 1, -128, 127)
        imag = map_range(clip_imag, -1, 1, -128, 127)
        # convert to 8 bits
        real = real.astype(np.byte)
        imag = imag.astype(np.byte)
        # convert back to complex64
        #output_items[0][:] = map_range(real,-128,127, -1, 1) + map_range(imag,-128,127, -1, 1)*1j
        output_items[0][:] = map_range(real.astype(np.float32),-128,127, -1, 1) + \
                                map_range(imag.astype(np.float32),-128,127, -1, 1)*1j
        #output_items[0][:] = real + imag*1j
        return len(output_items[0])
