#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2021 gr-kalvantula author.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#


import numpy as np
from gnuradio import gr

class correlation_normalized_float(gr.decim_block):
    """
    docstring for block correlation_find_shift
    """
    def __init__(self):
        gr.decim_block.__init__(self,
            name="correlation_normalized float",
            in_sig=[np.float32, np.float32],
            out_sig=[np.float32, np.float32, (np.float32, 2001)], decim=3000)

        self.samples_needed = 3000
        self.mid_sample = 1500
        self.max_shift = 1000
        self.correlate_size = 400
        np.set_printoptions(threshold=np.inf)


    def work(self, input_items, output_items):
        num_samples = len(input_items[0])

        used_sets = 0
        in0 = input_items[0]
        in1 = input_items[1]
        out = output_items[0]
        ph_out = output_items[1]
        dist_out = output_items[2]

        while num_samples >= self.samples_needed:
            start_index = self.samples_needed * used_sets
        
            
            #corr_array = np.concatenate((self.prev_buff1, in0[:1000]))

            corr_res = np.correlate(in0[start_index:start_index+self.samples_needed], in1[start_index + self.max_shift:start_index+self.samples_needed - self.max_shift])

            #corr_res = np.zeros(1500, dtype = np.complex64)
            #bb_correlation = np.correlate(in1[start_index + self.mid_sample : start_index + self.mid_sample + 2 * self.correlate_size],in1[start_index + self.mid_sample : start_index + self.mid_sample + 2 * self.correlate_size])
            #for i in range(self.mid_sample - self.max_shift, self.mid_sample + self.max_shift):
            #    corr_res[i - 750] = (np.correlate(in0[start_index + i:start_index + i+ 2 * self.max_shift],in1[start_index + 750:start_index + 1250]) / np.sqrt(np.correlate(in0[start_index + i:start_index + i+500],in0[start_index + i:start_index + i+500]) * bb_correlation))[0]

            #self.prev_buff1 = np.concatenate((self.prev_buff1[1000:], in0[:1000]))
            #self.prev_buff2[:] = in1[:1000]

            #print(corr_res)
            #print("max value: ", np.max(corr_res))
            #print("max arg: ", np.argmax(corr_res) - 750)
            #print("max value by arg: ", corr_res[np.argmax(corr_res)])

            max_ind = np.argmax(corr_res)
            out[used_sets] = max_ind - corr_res.size / 2
            ph_out[used_sets] = np.angle(corr_res[max_ind])
            dist_out[used_sets][:] = corr_res # - corr_res.size / 2
            #print(np.abs(corr_res)) 
            num_samples -= 3000

            used_sets += 1

        return used_sets

