import numpy as np
from gnuradio import gr
from scipy import signal as dsp
DEBUG = False

class cross_correlate(gr.sync_block):
    """
    outputs cross correlaton of two signals and the estimated lag in samples
    """
    def __init__(self, samp_size, wind_size, duty_factor):
        gr.sync_block.__init__(self,
            name="Complex to Hackrf",
            in_sig=[(np.complex64, samp_size),
                    (np.complex64, samp_size)],
            out_sig=[(np.complex64,samp_size-wind_size+1),
                    np.float32, np.float32]
        )
        self.samp_size = samp_size
        self.wind_size = wind_size
        self.duty_factor = duty_factor
        self.lags = lags = np.arange(-samp_size+ 1,samp_size)
        self.k = 1
        self.prev_corr = []
        self.prev_lag = 0
        self.prev_phase = 0

    def work(self, input_items, output_items):
        self.k-=1

        for i in range(len(input_items[0])):
            if self.k == 0: 
                self.k = self.duty_factor

                ref = input_items[0][i]
                signal = input_items[1][i]
                start = int((len(signal)/2.0) - (self.wind_size/2.0))
                corr = dsp.correlate(signal[start:start+self.wind_size], ref, 'valid')
#                if np.max(np.abs(corr)) != 0.0:
#                    corr /= np.max(corr)
                max_ind = np.argmax(np.abs(corr))

                #print (corr.shape, output_items[0].shape, corr.real.shape, len(input_items) , input_items[0].shape, input_items[1].shape)
                self.prev_lag = max_ind
                self.prev_phase = np.angle(corr[max_ind])
                self.prev_corr = corr

            output_items[0][i] = self.prev_corr
            output_items[1][i] = self.prev_lag
            output_items[2][i] = self.prev_phase

        return len(output_items[0])
