#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2021 gr-kalvantula author.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#


import numpy
from gnuradio import gr

class live_delay(gr.sync_block):
    """
    docstring for block live_delay
    """
    def __init__(self, delay_samples):
        gr.sync_block.__init__(self,
            name="live_delay",
            in_sig=[numpy.complex64],
            out_sig=[numpy.complex64])
        self.buffer = numpy.zeros(2000, dtype=numpy.complex64)
        self.delay_samples = delay_samples
        self.first_iter = True

    def set_delay_samples(self, delay_samples):
        self.delay_samples = delay_samples


    def work(self, input_items, output_items):
        input_len = len(input_items[0])
        if len(input_items[0]) > 2000:
            in0 = input_items[0]
            out = output_items[0]
            delay_samples_current = self.delay_samples
            
            
            out[:delay_samples_current] = self.buffer[2000-delay_samples_current:]
            out[delay_samples_current:] = in0[:input_len-delay_samples_current]
            self.buffer = in0[-2000:]
            return len(output_items[0])
        else:
            return 0

