import numpy as np
from gnuradio import gr


DEBUG = False

class phase_correction(gr.sync_block):
    """
    applies phase correction to complex signals
    from gr-doa
    d_ComplexGain(i) = gr_complex(1.0/GainEst, 0)*exp(gr_complex(0,-PhaseEst));
    """
    def __init__(self, gain, phase):
        gr.sync_block.__init__(self,
            name="Complex to Hackrf",
            in_sig=[np.complex64],
            out_sig=[np.complex64]
        )
        self.phase = phase
        self.gain = gain

    def work(self, input_items, output_items):
        output_items[0][:] = input_items[0] * (self.gain + 0j) * np.exp(0-self.phase*1j)
        return len(output_items[0])

    def set_phase(self, phase):
        if DEBUG:
            print("Setting phase: ", phase)
        self.phase = phase

    def set_gain(self, gain):
        if DEBUG:
            print("Setting gain: ", gain)
        self.gain = gain
