import numpy as np
from gnuradio import gr
from scipy import signal as dsp

class rotate_vectors(gr.sync_block):
    """
    outputs cross correlaton of two signals and the estimated lag in samples
    """
    def __init__(self, vec_size, rot1, rot2, rot3):
        gr.sync_block.__init__(self,
            name="Roll vectors",
            in_sig=[(np.float32, vec_size),
                    (np.float32, vec_size),
                    (np.float32, vec_size)],
            out_sig=[(np.float32, vec_size),
                    (np.float32, vec_size),
                    (np.float32, vec_size)],
        )
        self.rot1 = rot1
        self.rot2 = rot2
        self.rot3 = rot3
        self.k = 0
        self.mem1 = np.zeros(vec_size, np.float32)
        self.mem2 = np.zeros(vec_size, np.float32)
        self.mem3 = np.zeros(vec_size, np.float32)

    def work(self, input_items, output_items):
        for idx in range(len(input_items[0])):
            if self.k==100:
                self.mem1 = np.roll(input_items[0][idx], self.rot1)
                self.mem2 = np.roll(input_items[1][idx], self.rot2)
                self.mem3 = np.roll(input_items[2][idx], self.rot3)
                self.k = 0
            
            output_items[0][idx] = self.mem1
            output_items[1][idx] = self.mem2
            output_items[2][idx] = self.mem3
            self.k += 1

        return len(output_items[0])

    def set_rot1(self, new_rot):
        self.rot1 = new_rot

    def set_rot2(self, new_rot):
        self.rot2 = new_rot

    def set_rot3(self, new_rot):
        self.rot3 = new_rot