#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2021 gr-kalvantula author.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#


import numpy
from gnuradio import gr
from datetime import datetime
from time import time

class save_csv(gr.sync_block):
    def __init__(self, power_vlen, thresh, log_every, filepath):
        gr.sync_block.__init__(self,
            name="Log CSV",
            in_sig=[(numpy.float32, 360), (numpy.float32, power_vlen)],
            out_sig=[])
        self.thresh = thresh
        self.current_log_file = self.open_new_log_file()
        self.last_log_time = time()
        self.log_every = log_every
        self.k = 0
        self.header_line = 'timestamp,lat,lon,best_angle,power'


    def open_new_log_file(self):
        now = datetime.now()
        date_time = now.strftime("%Y-%m-%d_%H:%M:%S")
        filename = filepath + '/DOA_log_' + date_time + '.csv'
        self.current_log_file = open(filename, 'ba')
        self.current_log_file.write(self.header_line.encode('ascii'))
        

    def set_thresh(self, new_thresh):
        self.thresh = new_thresh


    def work(self, input_items, output_items):
        if self.last_log_time < time() - 300:
            self.current_log_file = self.open_new_log_file()
        
        for i in range(len(input_items[0])):
            if self.k >= self.log_every:
                data_array = np.array(time(), 20.1111, 50.1111, np.argmax(input_items[0][i]), 10*np.log10(np.average(input_items[1][i])))
                np.savetxt(self.current_log_file, data_array, delimiter=',', fmt=['%d', '%.10f', '%.10f', '%d', '%.2f'])
                k = 0
            k += 1
        
        return len(input_items[0])

