#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from PyQt5 import Qt
from gnuradio import gr
import numpy as np
from scipy.stats import circmean

class save_ph_diffs(gr.sync_block, Qt.QPushButton):

    def __init__(self, lbl, degs, freq, msm_idx, num_points, file_path):
        gr.sync_block.__init__(self, name="MsgPushButton", in_sig=[np.float32, np.float32, np.float32], out_sig=None)
        Qt.QPushButton.__init__(self, lbl)

        self.lbl = lbl
        self.degs = degs
        self.freq = freq
        self.num_points = num_points
        self.filehandle = open(file_path + "karakter_" + str(msm_idx) + "_sag_" + str(freq), 'w+')

        self.buf1 = np.zeros(num_points, dtype=np.float32)
        self.buf2 = np.zeros(num_points, dtype=np.float32)
        self.buf3 = np.zeros(num_points, dtype=np.float32)

        self.save_pending = False
        self.save_counter = 0

        self.setStyleSheet("")

        self.clicked[bool].connect(self.on_btn_clicked)

    def on_degs_changed(self, new_degs):
        self.degs = new_degs

    def do_actual_save(self):
        #hist_1, edges = np.histogram(self.buf1, bins=360, range=(-np.pi, np.pi))
        #hist_2, _     = np.histogram(self.buf2, bins=360, range=(-np.pi, np.pi))
        #hist_3, _     = np.histogram(self.buf3, bins=360, range=(-np.pi, np.pi))

        #prob_elem_1 = edges[np.argmax(hist_1)]
        #prob_elem_2 = edges[np.argmax(hist_2)]
        #prob_elem_3 = edges[np.argmax(hist_3)]

        prob_elem_1 = circmean(self.buf1, np.pi, -np.pi)
        prob_elem_2 = circmean(self.buf2, np.pi, -np.pi)
        prob_elem_3 = circmean(self.buf3, np.pi, -np.pi)

        string_line = str(self.degs) + "," + str(prob_elem_1) + "," + str(prob_elem_2) + "," + str(prob_elem_3) + "\n"

        self.filehandle.write(string_line)
        print(string_line)
        self.setEnabled(True)

    def on_btn_clicked(self, pressed):
        self.save_pending = True
        self.save_counter = 0
        self.setEnabled(False)

    def work(self, input_items, output_items):
        if self.save_pending:
            amount_of_new_data = input_items[0].size
            if amount_of_new_data > self.num_points:
                self.buf1 = input_items[0][-self.num_points:]
                self.buf2 = input_items[1][-self.num_points:]
                self.buf3 = input_items[2][-self.num_points:]
            else:
                self.buf1 = np.roll(self.buf1, -amount_of_new_data)
                self.buf2 = np.roll(self.buf2, -amount_of_new_data)
                self.buf3 = np.roll(self.buf3, -amount_of_new_data)

                self.buf2[-amount_of_new_data:] = input_items[0]
                self.buf3[-amount_of_new_data:] = input_items[1]
                self.buf1[-amount_of_new_data:] = input_items[2]
            self.save_counter += amount_of_new_data

            if self.save_counter >= self.num_points:
                self.save_pending = False
                self.do_actual_save()

        return input_items[0].size

    def set_degs(self, new_degs):
        self.degs = new_degs

    def stop(self):
        self.filehandle.close()
        return True