#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2021 gr-kalvantula author.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#


import numpy as np
from gnuradio import gr
import requests
import time

class send_ang_vec(gr.sync_block):
    """
    docstring for block send_ang_vector
    """
    def __init__(self, send_every, thresh, thresh_vlen, node_id, ip_addr):
        gr.sync_block.__init__(self,
            name="send_ang_vector",
            in_sig=[(np.float32, 360), (np.float32, thresh_vlen)],
            out_sig=[])
        self.thresh = thresh
        self.send_every = send_every
        self.counter = 0
        self.node_id = node_id
        self.ip_addr = ip_addr



    def work(self, input_items, output_items):
        for i in range(len(input_items[0])):
            self.counter += 1

            if(self.counter >= self.send_every):
                avg_pwr = np.average(input_items[1][i])
                try:
                    if avg_pwr > self.thresh:
                        r = requests.post("http://" + self.ip_addr + ":5678", data={"mtype":0, "id":self.node_id, "freq":31, "dist":input_items[0][i], "pow":avg_pwr, "time":int(time.time() * 1000)}, timeout=(0.001, 0.0001))
                        self.counter = 0
                    else:
                        r = requests.post("http://" + self.ip_addr + ":5678", data={"mtype":0, "id":self.node_id, "freq":31, "dist":np.zeros(360, dtype=np.float32), "pow":avg_pwr, "time":int(time.time() * 1000)}, timeout=(0.001, 0.0001))
                except ConnectionError:
                    #print("Cannot connect to server")
                    pass
                except ConnectionRefusedError:
                    #print("Cannot connect to server")
                    pass
                except ConnectionResetError:
                    pass
                    #print("Cannot connect to server")
                except requests.exceptions.ConnectionError:
                    pass
                    #print("Cannot connect to server")
                except requests.ConnectTimeout:
                    pass
                    #print("Connection timeout")
                except requests.ReadTimeout:
                    pass
                    #print("Connection timeout")
                self.counter = 0

        

        return len(input_items[0])


